# TP2AGaussStudents.py
"""----------------------------------------------------
Gauss reduction 
application to the resolution of a linear system
----------------------------------------------------"""
import numpy as np
#from numpy import linalg as LA
#import matplotlib.pyplot as plt

def GaussResolution(A,b):
    """
        Gauss resolution of the linear system Ax=b
        We assume that the matrix A is non singular
        Input : A is an array of shape (n,n)
                b is an array of shape (n, )
        Output : sol = array of shape (n, ) = the solution
    """
    u = np.shape(A) 
    n = u[0]
    # We concatenate matrix A and second member b
    bb = np.array([b])      # 2-dimensional array
    Ab = np.hstack((A,bb.T))
    # list for ordering variables
    var = np.arange(n)
    # GAUSS REDUCTION of matrix A & second member b
    for i in range(n-1):
        # we look for the max abs value in the bottom-right submatrix of A
        max = abs(Ab[i,i])
        imax = i
        jmax = i
        for ii in range(i,n):
            for jj in range(i,n):
                -----
                -----
                -----
                -----
        # we permute rows i and imax
        if imax != i:
            -----
            -----
            -----
        # we permute columns i and jmax
        if jmax != i:
            -----
            -----
            -----
            # we update the ranking of variables
            -----
            -----
            -----
        # finally, Gauss reduction :
        for ii in range(i+1,n):
            cf = Ab[ii,i] / Ab[i,i]
            -----
    
    # Resolution of the triangular system (bottom to top)
    xx = np.zeros(n)
    xx[n-1] = Ab[n-1,n] / Ab[n-1,n-1] 
    for k in range(n-2,-1,-1):
        -----
        -----
        -----
        -----
    
    # Re-ordering of the variables
    sol = np.zeros(n)
    for i in range(n):
        -----
        -----
    
    return sol

"""----------------------------------------------------
TESTS 
 --> we define a matrix and a vector b for the tests
 --> and we compare three methods :
         1) resolution with the inverse
         2) resolution with numpy.linalg.solve()
         3) resolution with the Gauss method
----------------------------------------------------"""
n = 10
#===========================================================
# Test matrix :
H = np.random.rand(n, n) #uniform distribution
H = H - 0.5
lc = np.linalg.cond(H)
#===========================================================

#===========================================================
# OR Hilbert matrix H(i,j) = 1. / (i+j+1)
H = np.zeros((n,n))
for i in range(n):
    for j in range(n):
        H[i,j] = 1. / (i+j+1)
lc = np.linalg.cond(H)
#===========================================================

# we choose a vector x as a solution : x = [1,2,3,...,n]
x  = np.arange(1,n+1)
# and then we compute the second member b
b = np.dot(H,x)

# 1) resolution with the inverse matrix of H
H1 = np.linalg.inv(H)
x1 = np.dot(H1,b) 
e1 = sum(abs(x1 - x)) / n     # error1

# 2) resolution with numpy.linalg.solve()
x2 = np.linalg.solve(H,b)
e2 = sum(abs(x2 - x)) / n     # error2

# 3) resolution with the Gauss method
x3 = GaussResolution(H,b)
e3 = sum(abs(x3 - x)) / n     # error3

# Comparizon between the 3 methods :
print("e1 = ",e1," inverse")
print("e2 = ",e2," linalg.solve")
print("e3 = ",e3," Gauss")

