# TP2AGramSchmidtStudents.py
"""---------------------------------------------------
1) Gram-Schmidt algorithm ...
2) to generate orthogonal matrices
------------------------------------------------------"""
import numpy as np
np.set_printoptions(precision=4) # no need of too much digits

"""-------------------
SOME USEFUL FUNCTIONS
----------------------"""
def innerProduct(u,v):
    """ u and v are vectors of same dimension (one-dimensional arrays 
        or list of numerical values)
        output : sum_i (u_i * v_i) = inner product of u and v
    """
    u = np.array(u)
    v = np.array(v)
    return sum(u*v)


def orthoProj(u,v):
    """
    Orthogonal projection of vector v on the line Vect(u)
    """
    uv = innerProduct(u,v)
    uu = innerProduct(u,u)
    return (uv / uu) * u


def norm(u):
    """ return the euclidean norm of vector u
    """
    uu = innerProduct(u,u)
    return np.sqrt(uu)

"""-------------------
GRAM-SCHMIDT
----------------------"""
def GramSchmidt(P):
    """
    Gram-Schmidt algorithm
    Input : P is a nonsingular square matrix of order n
            that can be viewed as n column vectors v_1,...,v_n 
            linearly independant in R^n
    Output : PO is the matrix whose column vectors e_1,...,e_n 
             are deduced from vectors v_i by Gram-Schmidt algorithm
             ==> the matrix PO is orthogonal
    """
    # initialization
    n,m = np.shape(P) # n=m
    PO = P
    # Gram-Schmidt orthogonalization loop
    for k in range(1,n):
        for j in range(k):
            ------
    # we normalize each column vector
    for k in range(n):
        ------
        ------
    return PO

"""-------------------
TESTS
----------------------"""
# we first define a 5x5 square matrix (with a list that we reshape)
P = [1,0,1,0,1,0,-1,1,0,1,-1,1,0,0,1,0,1,0,1,1,-1,-1,1,0,1.]
P = np.array(P)
P = P.reshape(5,5)

# determinant of matrix P :
dt = np.linalg.det(P) # dt=1.0
# ==> column vectors of P are linearly independant in R^5
# ==> they form a basis
# we determine the associated orthogonal basis with Gram-Schmidt algo
PO = GramSchmidt(P)
# we check :
np.dot(PO.T,PO)


"""-----------------------------
WE GENERATE ORTHOGONAL MATRICES
--------------------------------"""
n = 4       # order of orthogonal matrices
Nmax = 7    # number of orthogonal matrices that we will generate
ListPO = [] # list of orthogonal matrices

numberOfOrthoMatrices = 0
while numberOfOrthoMatrices < Nmax :
    P = np.random.uniform(0,1,n**2)
    P = P.reshape(n,n)
    # we test if matrix P is invertible
    dt = np.linalg.det(P) 
    if abs(dt) > 0.0001:
        # Gram-Schmidt orthogonalization
        PO = GramSchmidt(P)
        ListPO.append(PO)
        numberOfOrthoMatrices += 1
print(ListPO)

# condition number of each matrix in the list ListPO
from numpy import linalg as LA
for Q in ListPO:
    print(LA.cond(Q))


