# TP5BperiodicSplinesStudents.py

"""=========================================================
Interpolating periodic C2 cubic parametric spline :
    acquisition of a polygon,
    closure of this polygon,
    interpolation by a C2 cubic periodic parametric spline
    with chordal parameterization
============================================================"""
import numpy as np
import matplotlib.pyplot as plt

# Cubic Hermite basis over [0,1] :
def H0(t) :
    y = 1 - 3 * t**2 + 2 * t**3
    return y
def H1(t) :
    y = t - 2 * t**2 + t**3
    return y    
def H2(t) :
    y = - t**2 + t**3
    return y
def H3(t) :
    y = 3 * t**2 - 2 * t**3
    return y   

# Cubic Hermite interpolation over 2 points
def HermiteC1b(x0,y0,y0p,x1,y1,y1p):
    """ Cubic Hermite interpolation of order 1 over 2 points x0 < x1
        (interpolation of value + first derivative)
        Remark : no plotting in this version HermiteC1b()
        Input :
            x0,y0,y0p,x1,y1,y1p = Hermite data of order 1 (real values)
        Output :
            x = sampling of 100 values in interval [x0,x1]
            y = image of x by the cubic Hermite interpolant
    """
    x = np.linspace(x0,x1,100)
    h = x1 - x0
    t = (x - x0) / h
    y = y0 * H0(t) + y0p * h * H1(t) + y1p * h * H2(t) + y1 * H3(t)
    return x, y

# Cubic interpolating Hermite spline C1-C1 of n points
def splineC1C1(xi,yi,yip):
    """ Hermite C1 spline interpolating 'C1 data' (xi,yi,yip)
        --> the data are C1 (Hermite data of order one)
        --> the constructed spline is C1
        Input :
            xi,yi,yip = 3 arrays of size N (points, values, derivatives)
        Output :
            xs = sampling of 100*(N-1) values in [x_first,x_last]
            ys = image of xs by the the Hermite C1 spline
    """
    N = np.size(xi)
    xs = []
    ys = []
    for i in range(N-1):
        -----
        -----
        -----
    return xs, ys


def splineC2PeriodicNU(xi,yi):
    """ Periodic C2 cubic spline interpolating the (periodic) data (xi,yi) :
        --> the last yi-data is identical to the first one : yi[0] = yi[last]
        NU = NonUniform :
        --> arbitrary values xi in increasing order
        Input:
            xi,yi = vectors of data to be interpolated
        Output :
            xs = sampling of [xi_first, xi_last]
            ys = image of xs by the periodic spline
    """
    n = np.size(xi)
    h = xi[1:n] - xi[0:n-1]
    
    # construction of the spline matrix A of size (n-1)x(n-1) :
    -----
    -----
    -----
    -----
    -----
    -----
    -----
    -----
    -----
    
    # second member :
    b = np.zeros(n-1)
    b[0] = -----
    for k in range(1,n-1) :
        b[k] = -----
    b = 3 * b
    
    # determination of derivatives yip at points xi :
    yip = np.linalg.solve(A,b)
    yip = np.append(yip,yip[0]) # we duplicate the first derivative
    # evaluation of the spline :
    (xs, ys) = splineC1C1(xi,yi,yip)
    return (xs, ys)


# Mouse acquisition of a polygon
def PolygonAcquisitionClosed(color1,color2) :
    """ Mouse acquisition of a polygon : right click to stop
        the polygon is closed by joining the last point to the first one
        the first point is duplicated at the end of the polygon
        the returned values x, y are lists
    """
    x = []  # x and y are empty lists
    y = []
    coord = 0
    while coord != []:
        coord = plt.ginput(1, mouse_add=1, mouse_stop=3, mouse_pop=2)
        # coord is a list of tuples : coord = [(x,y)]
        if coord != []:
            xx = coord[0][0]
            yy = coord[0][1]
            plt.plot(xx,yy,color1,markersize=8)
            x.append(xx)
            y.append(yy)
            plt.draw()
            if len(x) > 1 :
                plt.plot([x[-2],x[-1]],[y[-2],y[-1]],color2)
    # closure of the polygon :
    plt.plot([x[0],x[-1]],[y[0],y[-1]],color2)
    x.append(x[0])
    y.append(y[0])
    return x,y




"""------------------------------------------------------
MAIN PROGRAM :
------------------------------------------------------"""
plt.cla()
fig = plt.figure(1,(8,8))
ax = fig.add_subplot(111)
xlim = (-10, 10)
ylim = (-10, 10)
ax.set_xlim(xlim)
ax.set_ylim(ylim)

# Acquisition of a closed polygon 
xi, yi = PolygonAcquisitionClosed('oc','c--')
N = np.size(xi) # number of distinct points + 1

# xi,yi are lists and have to be converted into arrays
xi = np.array(xi)
yi = np.array(yi)

# parameters domain :
a = 0; b = 1

# chordal parameterization :
tc = np.zeros(N)
for i in range(1,N) :
    -----
    -----
-----

# parametric interpolation according to the chordal parametrization tc
(tx,xs) = splineC2PeriodicNU(tc,xi)
(ty,ys) = splineC2PeriodicNU(tc,yi)
plt.plot(xs, ys,'r',lw=1,label='Periodic C2 cubic spline')

plt.legend(loc='best') 
